﻿#region

using System;
using System.Collections.Generic;
using Microsoft.Ceres.Optimizer.Engine;
using Microsoft.Ceres.Optimizer.GraphMatching;

#endregion

namespace Microsoft.Ceres.Optimizer
{
    /// <summary>
    /// Abstract class for transformation rules, i.e pre- and post-processors.
    /// </summary>
    public abstract class AbstractTransformationRule : ITransformationRule
    {
        #region ITransformationRule Members

        /// <summary>Before-graph-pattern triggering the rule.</summary>
        public abstract AbstractNodeMatcher Pattern { get; }

        /// <summary>
        /// Does the preprocessor need to be run several times until the graph no longer changes?
        /// Returns false by default.
        /// </summary>
        public virtual bool Iterative
        {
            get { return false; }
        }

        /// <summary>
        /// Called when the before-pattern matches the query graph.
        /// </summary>
        /// <param name="graphSearcher">Graphsearcher with information about the pattern matches.</param>
        /// <returns>If the query graph was changed.</returns>
        public abstract bool Fire(GraphSearcher graphSearcher);

        /// <summary>
        /// Some transformation rules need access to e.g. initialized rule binders.
        /// This is their chance.
        /// </summary>
        /// <param name="optimizationInstance">Current instance of QueryOptimization.</param>
        public virtual void Configure(QueryOptimization optimizationInstance)
        {
        }

        /// <summary>What other tranformation rules this rule depends on.</summary>
        public virtual IEnumerable<Type> DependsOn
        {
            // Return no dependencies by default.
            get { return new List<Type>(); }
        }

        #endregion
    }
}