﻿namespace Microsoft.Ceres.Optimizer.Engine
{
    /// <summary>
    /// Our simple state for plan sets, keeping track of cardinality and tuple size.
    /// </summary>
    public struct BasicPlanSetState : IPlanSetState
    {
        public BasicPlanSetState(double cardinality, double tupleSize) : this()
        {
            Cardinality = cardinality;
            TupleSize = tupleSize;
        }

        /// <summary>Cardinality of the plan set.</summary>
        public double Cardinality { get; set; }
        /// <summary>Tuple size in bytes.</summary>
        public double TupleSize { get; set; }

        /// <summary>Size of the result set, i.e. Cardinality * TupleSize.</summary>
        public double ResultSetSize
        {
            get { return Cardinality*TupleSize; }
        }

        /// <summary>Scales the tuple size by the specified factor, returning a new state instance.</summary>
        public BasicPlanSetState Scale(double factor)
        {
            return new BasicPlanSetState(Cardinality, TupleSize*factor);
        }

        /// <summary>Expands the tuple size with the specified number of bytes, returning a new state instance.</summary>
        public BasicPlanSetState Expand(double amount)
        {
            return new BasicPlanSetState(Cardinality, TupleSize + amount);
        }

        /// <summary>Filters (scales) the cardinality with the specified factor.</summary>
        public BasicPlanSetState Filter(double factor)
        {
            return new BasicPlanSetState(Cardinality*factor, TupleSize);
        }

        public override string ToString()
        {
            return string.Format("BasicPlanSetState(Cardinality: {0}, TupleSize: {1})",
                                 Cardinality, TupleSize);
        }
    }
}