﻿namespace Microsoft.Ceres.Optimizer.Engine
{
    /// <summary>How two cost instances relate to each other.</summary>
    public enum CostRelation
    {
        Better,
        Equal,
        Worse,
        /// <summary>None of the costs is better than the other and they are not equal.</summary>
        Unknown
    }

    /// <summary>Interface between the plan generator and the
    /// user-implemented cost model for the basic unit of cost.
    /// </summary>
    public interface ICost
    {
        /// <summary>
        /// Compares this cost instance to the specified instance.
        /// </summary>
        /// <param name="other">The cost instance to compare with.</param>
        /// <returns>Any CostRelation value describing the relation.</returns>
        CostRelation Compare(ICost other);
        /// <summary>
        /// Compares this cost instance to the specified instance, but
        /// Unknown is not allowed to be returned.
        /// </summary>
        /// <param name="other">The cost instance to compare with.</param>
        /// <returns>Any CostRelation value EXCEPT Unkown describing the relation.</returns>
        CostRelation CompareTotal(ICost other);
    }
}