﻿#region

using System;
using System.Runtime.Serialization;

#endregion

namespace Microsoft.Ceres.Optimizer.Engine
{
    /// <summary>
    /// Exception class for optimizer-related errors.
    /// </summary>
    [Serializable]
    public class OptimizerException : Exception
    {
        /// <summary>Default constructor.</summary>
        public OptimizerException()
        {
        }

        /// <summary>Constructs an exception with the specified message.</summary>
        /// <param name="message">Exception message.</param>
        public OptimizerException(string message) : base(message)
        {
        }

        /// <summary>Constructs an exception with the specified message and inner exception.</summary>
        /// <param name="message">Exception message.</param>
        /// <param name="innerException">Inner exception.</param>
        public OptimizerException(string message, Exception innerException) : base(message, innerException)
        {
        }

        /// <summary>Constructs an exception, building its message from a format string.</summary>
        /// <param name="format">Format string, <see cref="string.Format(string, object[])"/>.</param>
        /// <param name="args">Format arguments, <see cref="string.Format(string, object[])"/>.</param>
        public OptimizerException(string format, params object[] args) : base(string.Format(format, args))
        {
        }

        /// <summary>Constructor used for deserializing.</summary>
        /// <param name="info">Serialization info.</param>
        /// <param name="context">Streaming context.</param>
        public OptimizerException(SerializationInfo info, StreamingContext context) : base(info, context)
        {
        }
    }
}