﻿#region

using System;
using System.Collections.Generic;

#endregion

namespace Microsoft.Ceres.Optimizer.Engine.Orders
{
    /// <summary>
    /// Represents a node in the DFA.
    /// </summary>
    internal class DFANode : IComparable<DFANode>
    {
        private readonly LinkedList<Edge<DFANode>> edges = new LinkedList<Edge<DFANode>>();

        /// <summary>Outgoing edges from this node.</summary>
        public LinkedList<Edge<DFANode>> Edges
        {
            get { return edges; }
        }

        /// <summary>Set of NFA nodes this DFA node represents (from power set construction).</summary>
        public NestableHashSet<NFANode> Description { get; set; }
        /// <summary>The assigned Order instance for this state.</summary>
        public Order Order { get; set; }

        #region IComparable<DFANode> Members

        public int CompareTo(DFANode other)
        {
            return 0;
        }

        #endregion

        /// <summary>
        /// Adds an outgoing edge to this node.
        /// </summary>
        /// <param name="edge">The edge to add.</param>
        public void AddEdge(Edge<DFANode> edge)
        {
            edge.Origin = this;
            if (edge.Target != this)
            {
                edges.AddFirst(edge);
            }
        }

        /// <summary>
        /// If the NFA nodes of this node is a subset of the NFA of the specified node.
        /// </summary>
        /// <param name="other">The node to compare with.</param>
        /// <returns>True if it is, false if not.</returns>
        public bool IsSubsetOf(DFANode other)
        {
            return Description.IsSubsetOf(other.Description);
        }

        public override string ToString()
        {
            return "{" +
                   Utils.JoinEnumerable(Description, n => n.OrderDescription.ToString(), string.Empty, " ### ", true) +
                   "}";
        }
    }
}