﻿#region

using System.Collections.Generic;

#endregion

namespace Microsoft.Ceres.Optimizer.Engine.Orders
{
    /// <summary>
    /// Represents a functional dependency, FD for short.
    /// </summary>
    public class Dependency
    {
        public Dependency(int id, HashSet<string> from, HashSet<string> to, bool equal)
        {
            Id = id;
            From = from;
            To = to;
            Equal = equal;
        }

        public Dependency(int id, bool equal)
            : this(id, new HashSet<string>(), new HashSet<string>(), equal)
        {
        }

        /// <summary>Id of this FD.</summary>
        public int Id { get; set; }
        /// <summary>Left hand side properties.</summary>
        public HashSet<string> From { get; set; }
        /// <summary>Right hand side properties.</summary>
        public HashSet<string> To { get; set; }
        /// <summary>If this is an equality.</summary>
        public bool Equal { get; set; }

        public override string ToString()
        {
            return string.Format("D(Id:{0} {{{1}}} {2} {{{3}}})", Id,
                                 Utils.JoinEnumerable(From), Equal ? "=" : "->", Utils.JoinEnumerable(To));
        }

        public override bool Equals(object obj)
        {
            Dependency other = obj as Dependency;
            if (other == null)
                return false;
            return Id.Equals(other.Id);
        }

        public override int GetHashCode()
        {
            return Id.GetHashCode();
        }
    }
}