﻿#region

using System;

#endregion

namespace Microsoft.Ceres.Optimizer.Engine.Orders
{
    /// <summary>
    /// Represents and edge in either NFA or DFA.
    /// </summary>
    /// <typeparam name="T">Type of the nodes.</typeparam>
    internal class Edge<T> : IComparable<Edge<T>> where T : IComparable<T>
    {
        public Edge(T target, int label, string description)
        {
            Target = target;
            Label = label;
            Description = description;
        }

        /// <summary>Origin (from) node.</summary>
        public T Origin { get; set; }
        /// <summary>Target (to) node.</summary>
        public T Target { get; set; }
        /// <summary>Edge label.</summary>
        public int Label { get; set; }
        /// <summary>Friendly description for graphs.</summary>
        public string Description { get; set; }

        #region IComparable<Edge<T>> Members

        public int CompareTo(Edge<T> other)
        {
            if (Label != other.Label)
            {
                return Label.CompareTo(other.Label);
            }
            return Target.CompareTo(other.Target);
        }

        #endregion

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj) || !(obj is Edge<T>))
            {
                return false;
            }
            Edge<T> edge = (Edge<T>) obj;
            return ReferenceEquals(Target, edge.Target) && Label == edge.Label;
        }

        public override int GetHashCode()
        {
            return Target.GetHashCode() ^ Label.GetHashCode();
        }

        public override string ToString()
        {
            return string.Format("{0} {1} ### {2} ---> {3}",
                                 Label, Description, Origin, Target);
        }
    }
}