﻿#region

using System.Collections.Generic;
using System.Linq.Expressions;

#endregion

namespace Microsoft.Ceres.Optimizer
{
    public static class ExpressionExtensions
    {
        /* This is way too simple, but leaving it here for now, as we may
           find our needs are simpler than having to use ExpressionVisitor. */

        #region Delegates

        public delegate void ExpressionVisitor(Expression visitedExpression);

        #endregion

        public static IEnumerable<Expression> IteratePostorder(this Expression root)
        {
            Stack<Expression> stack = new Stack<Expression>();
            LinkedList<Expression> result = new LinkedList<Expression>();

            stack.Push(root);
            while (stack.Count > 0)
            {
                var node = stack.Pop();

                if (node is BinaryExpression)
                {
                    stack.Push(((BinaryExpression) node).Left);
                    stack.Push(((BinaryExpression) node).Right);
                }
                else if (node is UnaryExpression)
                {
                    stack.Push(((UnaryExpression) node).Operand);
                }
                else if (node is LambdaExpression)
                {
                    stack.Push(((LambdaExpression) node).Body);
                    continue;
                }
                result.AddFirst(node);
            }
            return result;
        }

        public static void TraversePostorder(this Expression expression, ExpressionVisitor visitor)
        {
            foreach (Expression e in expression.IteratePostorder())
            {
                visitor(e);
            }
        }
    }
}