﻿namespace Microsoft.Ceres.Optimizer.GraphMatching
{
    /// <summary>
    /// Matcher that returns true as long as all nodes so far have matched the sub-pattern.
    /// Whenever a node does not match the sub-pattern, the pattern will continue to return false.
    /// </summary>
    public class AllMatcher : MatcherWrapper
    {
        private bool allTrue;

        public AllMatcher(AbstractNodeMatcher pattern)
        {
            Name = "All";
            AddChild(pattern);
            allTrue = true;
        }

        public override bool HasMatched
        {
            get { return allTrue && Pattern.HasMatched; }
        }

        public override bool Search(Node node)
        {
            if (!allTrue)
                return false;

            if (! Pattern.Search(node))
                return allTrue = false;

            MatchedNodes.Add(node);
            return true;
        }

        public override void Reset()
        {
            base.Reset();
            allTrue = true;
        }
    }
}