﻿#region

using System.Collections.Generic;
using System.Linq;

#endregion

namespace Microsoft.Ceres.Optimizer.GraphMatching
{
    /// <summary>
    /// Matches nodes that match all sub-patterns.
    /// </summary>
    public class AndMatcher : AbstractCompoundMatcher
    {
        public AndMatcher(IEnumerable<AbstractNodeMatcher> patterns)
        {
            AddChildren(patterns.Cast<Node>());
        }

        public AndMatcher(params AbstractNodeMatcher[] types)
            : this(types.AsEnumerable())
        {
        }

        public override bool Search(Node node)
        {
            foreach (AbstractNodeMatcher pattern in SubPatterns)
            {
                if (!pattern.Search(node))
                    return HasMatched = false;
            }
            MatchedNodes.Add(node);
            return HasMatched = true;
        }

        public static AndMatcher operator &(AndMatcher a, AbstractNodeMatcher b)
        {
            a.AddChild(b);
            return a;
        }

        public static AndMatcher operator &(AbstractNodeMatcher a, AndMatcher b)
        {
            return b & a;
        }

        public override string ToString()
        {
            return Utils.JoinEnumerable(Children, " & ");
        }
    }
}