﻿#region

using System.Collections.Generic;
using System.Linq;

#endregion

namespace Microsoft.Ceres.Optimizer.GraphMatching
{
    /// <summary>
    /// Matches nodes that match at least one sub-pattern.
    /// </summary>
    internal class OrMatcher : AbstractCompoundMatcher
    {
        public OrMatcher(params AbstractNodeMatcher[] patterns)
            : this(patterns.AsEnumerable())
        {
        }

        public OrMatcher(IEnumerable<AbstractNodeMatcher> patterns)
        {
            AddChildren(patterns.Cast<Node>());
        }

        public override bool Search(Node node)
        {
            foreach (IGraphPattern matcher in SubPatterns)
            {
                if (matcher.Search(node))
                {
                    MatchedNodes.Add(node);
                    return HasMatched = true;
                }
            }
            return HasMatched = false;
        }

        public static OrMatcher operator |(OrMatcher a, AbstractNodeMatcher b)
        {
            a.AddChild(b);
            return a;
        }

        public static OrMatcher operator |(AbstractNodeMatcher a, OrMatcher b)
        {
            return b | a;
        }

        public override string ToString()
        {
            return Utils.JoinEnumerable(Children, " | ");
        }
    }
}