﻿namespace Microsoft.Ceres.Optimizer.GraphMatching
{
    /// <summary>
    /// Matches as long as at least min and at most max input nodes have matched the sub-pattern.
    /// </summary>
    public class RangeQuantifier : MatcherWrapper
    {
        protected int matches;
        public int max, min;

        public RangeQuantifier(AbstractNodeMatcher pattern, int min, int max)
        {
            AddChild(pattern);
            this.min = min;
            this.max = max;
            matches = 0;
        }

        public override bool HasMatched
        {
            get
            {
                max = max >= 0 ? max : int.MaxValue;
                return matches >= min && matches <= max;
            }
            set { }
        }

        public override bool Search(Node node)
        {
            if (!Pattern.Search(node))
                return false;

            MatchedNodes.Add(node);
            matches++;
            return true;
        }

        public override void Reset()
        {
            base.Reset();
            matches = 0;
        }

        public override string ToString()
        {
            return string.Format("<{0},{1}>({2})", min, max, Pattern.ToString());
        }
    }
}