﻿#region

using Microsoft.Ceres.Optimizer.Engine;
using Microsoft.Ceres.Optimizer.Engine.Orders;

#endregion

namespace Microsoft.Ceres.Optimizer
{
    /// <summary>
    /// Interface between the query optimizer and the system catalog.
    /// </summary>
    public interface ISystemMetadata
    {
        /// <summary>
        /// Gets statistics (cardinality, tuple size) for an index.
        /// </summary>
        /// <param name="indexName">Name of the index.</param>
        /// <param name="lookupWord">Lookup word in the index.</param>
        BasicPlanSetState GetIndexStats(string indexName, string lookupWord);
        /// <summary>
        /// Gets schema information (attributes etc) for an index.
        /// </summary>
        /// <param name="indexName">name of the index.</param>
        /// <param name="lookupWord">Lookup word in the index.</param>
        RecordDescriptor GetIndexSchema(string indexName, string lookupWord);
        /// <summary>
        /// Gets index ordering (clustering) information for an index.
        /// </summary>
        /// <param name="indexName">name of the index.</param>
        OrderDescription GetIndexOrder(string indexName);
    }
}