﻿#region

using System;
using System.Collections.Generic;
using Microsoft.Ceres.Optimizer.Engine;
using Microsoft.Ceres.Optimizer.GraphMatching;

#endregion

namespace Microsoft.Ceres.Optimizer
{
    /// <summary>
    /// Common interface for transformation rules, i.e pre- and post-processors.
    /// </summary>
    public interface ITransformationRule
    {
        /// <summary>What other tranformation rules this rule depends on.</summary>
        IEnumerable<Type> DependsOn { get; }
        /// <summary>
        /// Does the preprocessor need to be run several times until the graph no longer changes?
        /// </summary>
        bool Iterative { get; }
        /// <summary>Before-graph-pattern triggering the rule.</summary>
        AbstractNodeMatcher Pattern { get; }
        /// <summary>
        /// Called when the before-pattern matches the query graph.
        /// </summary>
        /// <param name="graphSearcher">Graphsearcher with information about the pattern matches.</param>
        /// <returns>If the query graph was changed.</returns>
        bool Fire(GraphSearcher graphSearcher);
        /// <summary>
        /// Some transformation rules need access to e.g. initialized rule binders.
        /// This is their chance.
        /// </summary>
        /// <param name="optimizationInstance">Current instance of QueryOptimization.</param>
        void Configure(QueryOptimization optimizationInstance);
    }
}