﻿namespace Microsoft.Ceres.Optimizer
{
    /// <summary>
    /// Represents a pair of two values with Types T and Y.
    /// Provides implementations of Equals, GetHashCode and equality operators.
    /// </summary>
    public class Pair<T, Y>
    {
        public Pair(T first, Y second)
        {
            First = first;
            Second = second;
        }

        /// <summary>Value of the first item.</summary>
        public virtual T First { get; set; }
        /// <summary>Value of the second item.</summary>
        public virtual Y Second { get; set; }

        public override string ToString()
        {
            return Utils.NullFormat("<{0},{1}>", First, Second);
        }

        public override int GetHashCode()
        {
            return (First != null ? First.GetHashCode() : 0) ^ (Second != null ? Second.GetHashCode() : 0);
        }

        public override bool Equals(object obj)
        {
            Pair<T, Y> other = obj as Pair<T, Y>;
            if (other == null)
                return false;

            bool firstAreEqual = (First == null && other.First == null) || (First != null && First.Equals(other.First));
            bool secondAreEqual = (Second == null && other.Second == null) ||
                                  (Second != null && Second.Equals(other.Second));

            return firstAreEqual && secondAreEqual;
        }

        public static bool operator ==(Pair<T, Y> a, Pair<T, Y> b)
        {
            return ReferenceEquals(a, b) || (!ReferenceEquals(null, a) && a.Equals(b));
        }

        public static bool operator !=(Pair<T, Y> a, Pair<T, Y> b)
        {
            return !(a == b);
        }
    }
}