﻿#region

using System;
using System.Collections.Generic;
using Microsoft.Ceres.Evaluation.Operators.BuiltIn.Basic;
using Microsoft.Ceres.Optimizer.GraphMatching;

#endregion

namespace Microsoft.Ceres.Optimizer.Postprocessors
{
    /// <summary>
    /// Only Copy in MARS can have multiple outputs. In the search phase,
    /// the Copy operator does not exist. Thus, these must be inserted.
    /// </summary>
    [Postprocessor]
    public class CopyInserter : AbstractTransformationRule
    {
        public override IEnumerable<Type> DependsOn
        {
            // TODO: Actually, FieldAnnotator should declare that it precedes CopyInserter, so CopyInserter need not know about it...
            get { yield return typeof (FieldAnnotator); }
        }

        public override AbstractNodeMatcher Pattern
        {
            get { return Match.AnyNode().WithMultipleParents().GroupAs("branchingNodes"); }
        }

        public override bool Fire(GraphSearcher graphSearcher)
        {
            foreach (OperatorNode node in graphSearcher.Groups["branchingNodes"])
            {
                OperatorNode copyNode = new OperatorNode(typeof (CopyOperator));
                node.InsertBetweenAllParents(copyNode);
            }
            return true;
        }
    }
}