﻿#region

using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Ceres.Evaluation.Operators;
using Microsoft.Ceres.Evaluation.Operators.BuiltIn.Basic;
using Microsoft.Ceres.Optimizer.GraphMatching;

#endregion

namespace Microsoft.Ceres.Optimizer.Postprocessors
{
    /// <summary>
    /// Insert a map operator that adds a column "OptimizerWasHere" to the output
    /// to prove that the optimizer touched the query.
    /// </summary>
    //[Postprocessor]
    public class OptimizerWasHere : AbstractTransformationRule
    {
        public override IEnumerable<Type> DependsOn
        {
            get { yield return typeof (FieldAnnotator); }
        }

        public override AbstractNodeMatcher Pattern
        {
            get { return Match.NodeOfType("OutputOperator").GroupAs("output"); }
        }

        public override bool Fire(GraphSearcher graphSearcher)
        {
            OperatorNode outputOperator = (OperatorNode) graphSearcher.Groups["output"].Single();
            foreach (OperatorNode query in new List<Node>(outputOperator.Children))
            {
                OperatorNode mapNode = new OperatorNode(typeof (MapOperator));
                mapNode["RetainInput"] = true;
                mapNode["ParameterMap"] = new Dictionary<IdentifierProperty, ExpressionProperty>
                                              {
                                                  {
                                                      new IdentifierProperty("OptimizerWasHere"),
                                                      new ExpressionProperty("\"Oh yes, I was!\"")
                                                      }
                                              };
                query.InsertBetweenAllParents(mapNode);
            }
            return true;
        }
    }
}