﻿#region

using System.Linq;
using Microsoft.Ceres.Optimizer.Rules.Infrastructure;

#endregion

namespace Microsoft.Ceres.Optimizer.Preprocessors
{
    /// <summary>
    /// Sets the operator's ``behaviour''  ---
    /// that is whether the operator changes the record structure, the amount
    /// of tuples, its ordering/grouping, etc. For example, a selection
    /// operator changes the tuple amount, but not the ordering. A map
    /// operator can change the record structure, but not the amount or ordering.
    /// A group operator potentially changes both tuple amount, ordering,
    /// grouping and record structure.
    /// </summary>
    [Preprocessor]
    public class BehaviourMapperAnnotator : AbstractAnnotator<IBehaviorMapper>
    {
        public override void Annotate(OperatorNode root)
        {
            foreach (OperatorNode node in root.GetTopologicalOrdering().Cast<OperatorNode>())
                if (mapperForType.ContainsKey(node.OperatorType))
                    mapperForType[node.OperatorType].SetBehaviour(node);
        }
    }
}