﻿#region

using System;
using System.Collections.Generic;
using Microsoft.Ceres.Optimizer.Rules.Infrastructure;

#endregion

namespace Microsoft.Ceres.Optimizer.Preprocessors
{
    /// <summary>
    /// Sets the operator's dependencies.
    /// </summary>
    [Preprocessor]
    public class DependencyAnnotator : AbstractAnnotator<IDependencyMapper>
    {
        public override IEnumerable<Type> DependsOn
        {
            get { return new List<Type> {typeof (ProducesAnnotator), typeof (EquivalenceTransformer)}; }
        }

        public override void Annotate(OperatorNode root)
        {
            foreach (OperatorNode node in root.GetTopologicalOrdering())
            {
                if (! mapperForType.ContainsKey(node.OperatorType))
                    continue;

                mapperForType[node.OperatorType].SetDependencies(node);

                // Add transitive dependencies to avoid unreachable plans
                foreach (HashSet<NodeAttribute> childDependencies in node.Dependencies)
                {
                    HashSet<OperatorNode> dependedOnNodes = new HashSet<OperatorNode>();
                    foreach (NodeAttribute dependency in childDependencies)
                    {
                        dependedOnNodes.Add(dependency.Node);
                        dependency.Node.Dependers.Add(node);
                    }
                    foreach (OperatorNode dependedOnNode in dependedOnNodes)
                    {
                        foreach (OperatorNode transitiveNode in dependedOnNode.GetAllNodeDependencies())
                        {
                            childDependencies.Add(new NodeAttribute(transitiveNode, null));
                            transitiveNode.Dependers.Add(node);
                        }
                    }
                }
            }
        }
    }
}