﻿#region

using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Ceres.Optimizer.Engine;
using Microsoft.Ceres.Optimizer.Rules.Infrastructure;

#endregion

namespace Microsoft.Ceres.Optimizer.Preprocessors
{
    /// <summary>
    /// Marks which attributes are deemed
    /// equivalent. For example, an equi-join of two attributes will induce
    /// that the two are equivalent. 
    /// </summary>
    [Preprocessor]
    public class EquivalenceTransformer : AbstractAnnotator<IEquivalenceMapper>
    {
        public override IEnumerable<Type> DependsOn
        {
            get { yield return typeof (ProducesAnnotator); }
        }

        public override void Annotate(OperatorNode root)
        {
            Dictionary<NodeAttribute, NestableHashSet<NodeAttribute>> equivalenceMap =
                new Dictionary<NodeAttribute, NestableHashSet<NodeAttribute>>();

            foreach (OperatorNode node in root.GetTopologicalOrdering().Reverse().Cast<OperatorNode>())
            {
                if (!mapperForType.ContainsKey(node.OperatorType))
                    continue;

                foreach (HashSet<NodeAttribute> equivalenceSet in mapperForType[node.OperatorType].GetEquivalences(node)
                    )
                {
                    NodeAttribute foundPair = null;
                    foreach (NodeAttribute attribute in equivalenceSet)
                    {
                        if (equivalenceMap.ContainsKey(attribute))
                        {
                            foundPair = attribute;
                            break;
                        }
                    }
                    if (foundPair != null)
                    {
                        equivalenceMap[foundPair].UnionWith(equivalenceSet);
                    }
                    else
                    {
                        foundPair = equivalenceSet.ElementAt(0);
                        equivalenceMap[foundPair] = new NestableHashSet<NodeAttribute>(equivalenceSet);
                    }

                    foreach (NodeAttribute attribute in equivalenceSet)
                    {
                        equivalenceMap[attribute] = equivalenceMap[foundPair];
                    }
                }
            }

            // TODO: The usage of HashSets as keys is *very* inefficient. Not a priority right now, tho.
            // Replace the attributes with a representative of the equivalence class.
            //Dictionary<NestableHashSet<NodeAttribute>, NodeAttribute> equivalenceRepresentativeMap = new Dictionary<NestableHashSet<NodeAttribute>, NodeAttribute>();
            //foreach (NestableHashSet<NodeAttribute> equivalenceSet in equivalenceMap.Values)
            //{
            //    if (equivalenceRepresentativeMap.ContainsKey(equivalenceSet))
            //        continue;

            //    NodeAttribute representative = 
            //}
        }
    }
}