﻿#region

using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Ceres.Evaluation.Operators.BuiltIn.Join;
using Microsoft.Ceres.Optimizer.Engine;
using Microsoft.Ceres.Optimizer.GraphMatching;
using Microsoft.Ceres.Optimizer.Operators;

#endregion

namespace Microsoft.Ceres.Optimizer.Preprocessors
{
    [Preprocessor]
    public class LogicalJoinTransformer : AbstractTransformationRule
    {
        public override bool Iterative
        {
            get { return false; }
        }

        public override IEnumerable<Type> DependsOn
        {
            get { return new List<Type>(); }
        }

        public override AbstractNodeMatcher Pattern
        {
            get
            {
                return
                    (Match.NodeOfType("HybridHashJoinOperator") | Match.NodeOfType("MergeJoinOperator")).GroupAs("join");
            }
        }

        public override bool Fire(GraphSearcher graphSearcher)
        {
            List<Node> joins = graphSearcher.Groups["join"];
            foreach (OperatorNode join in joins)
            {
                IList<string> joinKey = new List<string>();

                // To convert into a logical join, we need to determine the proper join keys.
                // TODO: For simplicity, we've currently limited ourself to having non-composite keys.
                if (join.OperatorType == typeof (HybridHashJoinOperator))
                {
                    IList<IList<string>> joinKeys = (IList<IList<string>>) join["JoinKeys"];
                    if (joinKeys.Count != 1)
                        throw new OptimizerException("Only single join keys are accepted in the prototype.");
                    joinKey = joinKeys.Single();
                }
                else if (join.OperatorType == typeof (MergeJoinOperator))
                {
                    int joinPrefix = (int) join["JoinPrefix"];
                    if (joinPrefix != 1)
                        throw new OptimizerException("Only single join keys are accepted in the prototype.");

                    for (int i = 0; i < join.Children.Count; i++)
                        joinKey.Add(((OperatorNode) join.Children[i]).OutputTypeDescriptor.RecordType[0].Name);
                }

                // TODO: Fix name confusion, should be using "JoinKeys" all over.
                join["JoinKey"] = joinKey;
                join.OperatorType = typeof (JoinOperator);
            }
            return true;
        }
    }
}