﻿#region

using System;
using System.Diagnostics;
using Microsoft.Ceres.Optimizer.GraphMatching;

#endregion

namespace Microsoft.Ceres.Optimizer.Preprocessors
{
    /// <summary>
    /// Merges sort and trim operators with orderpreserving and setpreserving operators between them.
    /// </summary>
    public class MergeTrimSort : AbstractTransformationRule
    {
        public override AbstractNodeMatcher Pattern
        {
            get
            {
                return Match.NodeOfType("TrimOperator").GroupAs("trim")
                    .WithDescendant(
                    (new AnythingMatcher()).WithSingleParent(),
                    // TODO: Should be NodeBehaviourMatcher.All(OperatorBehaviour.SetPreserving | OperatorBehaviour.OrderPreserving)
                    Match.NodeOfType("SortOperator").GroupAs("sort")
                    );
            }
        }

        public override bool Iterative
        {
            get { return true; }
        }

        public override bool Fire(GraphSearcher graphSearcher)
        {
            Node trimOperator = graphSearcher.Groups["trim"][0];
            Node sortOperator = graphSearcher.Groups["sort"][0];

            // Remove the trim operator from the graph.
            trimOperator.Extract();

            // Then modify the sort operator's properties to account for the trim.
            int trimOffset = (int) trimOperator["offset"];
            int sortOffset = (int) sortOperator["offset"];
            int finalOffset = trimOffset + sortOffset;

            int trimHitCount = (int) trimOperator["hitcount"];
            int sortHitCount = (int) sortOperator["hitcount"];
            int finalHitCount = -1;

            Debug.Assert(sortHitCount != 0 || trimHitCount != 0,
                         "This query will not return any results and should be replaced with a NOOP.");

            // The hit count will be no more than that of the trim.
            if (trimHitCount > 0)
                finalHitCount = trimHitCount;

            /* If the sort also has a limit, we want the lowest of the two, but we must also account for
             * an offset, if any. */
            if (sortHitCount > 0)
                finalHitCount = Math.Min(finalHitCount, Math.Max(sortHitCount - trimOffset, 0));

            sortOperator["offset"] = finalOffset;
            sortOperator["hitcount"] = finalHitCount;

            return true;
        }
    }
}