﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Ceres.Optimizer.GraphMatching;
using System.Diagnostics;
using Microsoft.Ceres.Evaluation.EvalBasicOp;

namespace Microsoft.Ceres.Optimizer.Preprocessors
{
    public class MergeTrimSortPreprocessor : AbstractPreprocessor
    {
        public override AbstractNodeMatcher Pattern
        {
            get
            {
                return (new NodeTypeMatcher("TrimOperator"))
                            .GroupAs("trim")
                            .WithChildren(
                                new ZeroOrMore(
                                    NodeBehaviourMatcher.All(OperatorBehaviour.SetPreserving | OperatorBehaviour.DataPreserving)
                                )
                                .WithChildren(
                                    (new NodeTypeMatcher("SortOperator")).GroupAs("sort")
                                )
                            );
            }
        }

        public override void Fire(PatternMatch match)
        {
            Node trimOperator = match.Groups["trim"].OnlyMatch;
            Node sortOperator = match.Groups["sort"].OnlyMatch;

            trimOperator.Parent.Children.Remove(trimOperator);
            trimOperator.Parent.Children.Add(trimOperator.OnlyChild);
            trimOperator.OnlyChild.Parent = trimOperator.Parent;

            int trimOffset = (int)trimOperator["offset"];
            int sortOffset = (int)sortOperator["offset"];
            int finalOffset = trimOffset + sortOffset;

            int trimHitCount = (int)trimOperator["hitcount"];
            int sortHitCount = (int)sortOperator["hitcount"];
            int finalHitCount = -1;

            Debug.Assert(sortHitCount != 0 || trimHitCount != 0,
                         "This query will not return any results and should be replaced with a NOOP.");

            if(trimHitCount > 0)
                finalHitCount = trimHitCount;

            if(sortHitCount > 0)
                finalHitCount = Math.Min(finalHitCount, Math.Max(sortHitCount - trimOffset, 0));

            sortOperator["offset"] = finalOffset;
            sortOperator["hitcount"] = finalHitCount;

        }

        public override bool Iterative { get { return true; } }
    }
}
