﻿#region

using System.Collections.Generic;
using System.Diagnostics;
using Microsoft.Ceres.Optimizer.GraphMatching;

#endregion

namespace Microsoft.Ceres.Optimizer.Preprocessors
{
    
    // TODO: We're unsure whether we want this thing at all. See report.
    /// <summary>
    /// Splits n-way (n >= 3) joins into binary joins.
    /// </summary>
    //[Preprocessor]
    public class MultiJoinSplitter : AbstractTransformationRule
    {
        public override AbstractNodeMatcher Pattern
        {
            get
            {
                return Match.NodeOfType("Join").GroupAs("join")
                    .WithAllChildren(
                    new RangeQuantifier(Match.AnyNode(), 3, -1)
                    );
            }
        }

        public override bool Iterative
        {
            get { return true; }
        }

        public void RewriteJoin(OperatorNode join)
        {
            Debug.Assert(join.Children.Count >= 3, "Expected join to be rewritten to have at least three children.");
            List<Node> joinedRelations = join.Children;
            Stack<OperatorNode> replacementJoins = new Stack<OperatorNode>();

            OperatorNode currentParent = join;
            foreach (Node joinedRelation in joinedRelations.GetRange(1, joinedRelations.Count - 1))
            {
                // We split into binary joins.
                currentParent.Properties["RequiredPrefix"] = 2;

                join.RemoveChild(joinedRelation);
                OperatorNode replacementJoin = (OperatorNode) currentParent.Clone();

                replacementJoin.Children.Clear();
                replacementJoin.Parents.Clear();

                replacementJoin.AddChild(joinedRelation);
                currentParent.AddChild(replacementJoin);

                replacementJoins.Push(replacementJoin);
                currentParent = replacementJoin;
            }

            currentParent.Properties["RequiredPrefix"] = 2;

            currentParent.Extract();
            replacementJoins.Pop();
        }

        public override bool Fire(GraphSearcher graphSearcher)
        {
            foreach (OperatorNode join in graphSearcher.Groups["join"])
            {
                RewriteJoin(join);
            }
            return true;
        }
    }
}