﻿#region

using System;
using System.Collections.Generic;
using Microsoft.Ceres.Optimizer.Engine;
using Microsoft.Ceres.Optimizer.Rules.Infrastructure;

#endregion

namespace Microsoft.Ceres.Optimizer.Preprocessors
{
    /// <summary>
    /// Maps the origins of attributes used in
    /// operators. The purpose is to always know where an attribute
    /// originates, to be able to correctly map operator dependencies. Also,
    /// attributes in expressions (such as in Map and
    /// Select) are altered to unambiguously refer to the attribute
    /// of a certain node using NodeAttribute instances instead of names.
    /// </summary>
    [Preprocessor]
    public class ProducesAnnotator : AbstractAnnotator<IProducesMapper>
    {
        public override IEnumerable<Type> DependsOn
        {
            get
            {
                return new List<Type>
                           {typeof (CopyRemover), typeof (LogicalJoinTransformer), typeof (BehaviourMapperAnnotator)};
            }
        }

        public override void Annotate(OperatorNode root)
        {
            foreach (OperatorNode node in root.GetTopologicalOrdering())
                if (mapperForType.ContainsKey(node.OperatorType))
                    mapperForType[node.OperatorType].SetProduces(queryOptimization, node);
                else
                    // TODO: Perhaps check this on optimizer initialization instead...
                    throw new OptimizerException("Operator type {0} does not have a produces mapper.", node.OperatorType);
        }
    }
}