﻿#region

using System;
using System.Collections.Generic;
using System.Reflection;

#endregion

namespace Microsoft.Ceres.Optimizer
{
    /// <summary>
    /// Utility class for reflecting types implementing plugins.
    /// </summary>
    internal static class ReflectionPluginHelper
    {
        /// <summary>Gets all types in the accessible assemblies tagged with the specified attribute.</summary>
        /// <param name="attributeType">Type of the attribute to look for.</param>
        /// <returns>A enumerable of the found types.</returns>
        public static IEnumerable<Type> GetTypesWithAttribute(Type attributeType)
        {
            foreach (Assembly assembly in AppDomain.CurrentDomain.GetAssemblies())
            {
                // The attribute need to be tagged with the OptimizerPluginsAttribute.
                if (Attribute.GetCustomAttribute(assembly, typeof (OptimizerPluginsAttribute)) == null)
                    continue;

                foreach (Type type in assembly.GetTypes())
                {
                    if (Attribute.GetCustomAttribute(type, attributeType, true) != null)
                        yield return type;
                }
            }
        }
    }
}