﻿#region

using System.Collections.Generic;
using Microsoft.Ceres.Optimizer.Engine;
using Microsoft.Ceres.Optimizer.Rules.Infrastructure;

#endregion

namespace Microsoft.Ceres.Optimizer.Rules
{
    /// <summary>Base class for base rules.</summary>
    public abstract class AbstractBaseRule : AbstractRule, IBaseRule
    {
        public AbstractBaseRule(QueryOptimization queryOptimization)
            : base(queryOptimization)
        {
        }

        #region IBaseRule Members

        /// <summary>Produced properties for this rule.</summary>
        public BitSetProxy Produced { get; set; }

        /// <summary>Filter for this rule, equal to produced properties.</summary>
        public BitSet Filter
        {
            get { return Produced.BitSet; }
        }

        /// <summary>Id of this rule.</summary>
        public int Id { get; set; }

        /// <summary>
        /// Determines if this and the other rule is structurally identical, i.e. if they have
        /// the same configured properties, the same join keys etc.
        /// </summary>
        /// <param name="other">Rule to compare with.</param>
        /// <returns>If the rules are structurally identical.</returns>
        public abstract bool StructurallyIdentical(IProducerRule other);

        /// <summary>
        /// This method will be called if this rule is determined to be share equivalent with another rule.
        /// It should map any attributes the other rule produces to be share equivalent to the corresponding attributes
        /// produced by this rule.
        /// </summary>
        /// <param name="from">The rule to map attributes from.</param>
        /// <param name="equivalenceClassAttributeMappings">The dictionary the mappings should be added to.</param>
        public abstract void MapShareEquivalentAttributes(IProducerRule from,
                                                          Dictionary<NodeAttribute, NodeAttribute>
                                                              EquivalenceClassAttributeMappings);

        /// <summary>
        /// Initialize a plan set with properties from the rule, like tuple size, cardinality etc.
        /// </summary>
        /// <param name="plans">The plan set to initialize.</param>
        public abstract void Initialize(PlanSet plans);

        #endregion
    }
}