﻿#region

using System;
using System.Collections.Generic;
using Microsoft.Ceres.Optimizer.Engine;
using Microsoft.Ceres.Optimizer.GraphMatching;
using Microsoft.Ceres.Optimizer.Rules.Infrastructure;

#endregion

namespace Microsoft.Ceres.Optimizer.Rules
{
    /// <summary>
    /// Abstract rule binder for filtering operators.
    /// Will set OperatorBehaviour.DataPreserving and OperatorBehaviour.OrderPreserving
    /// as well as passing record decriptors through.
    /// </summary>
    /// <typeparam name="RuleType"></typeparam>
    public abstract class AbstractFilterRuleBinder<RuleType> : AbstractRuleBinder<RuleType>, IProducesMapper,
                                                               IFieldMapper, IBehaviorMapper
        where RuleType : IProducerRule
    {
        #region IBehaviorMapper Members

        public void SetBehaviour(OperatorNode node)
        {
            // Filters should not modify the records nor the order.
            node.Behaviour = OperatorBehaviour.DataPreserving | OperatorBehaviour.OrderPreserving;
        }

        #endregion

        #region IFieldMapper Members

        public virtual void SetFields(QueryOptimization queryOptimization, OperatorNode node)
        {
            // Filters should just pass through attributes, without altering them.
            node.PassthroughFieldNames();
        }

        #endregion

        #region IProducesMapper Members

        public abstract IEnumerable<Type> MapperFor { get; }

        public virtual void SetProduces(QueryOptimization queryOptimization, OperatorNode node)
        {
            // Filters do not produce new attributes.
            node.PassthroughInputAttributeOrigins();
        }

        #endregion
    }
}