﻿#region

using System.Linq;
using Microsoft.Ceres.Optimizer.Engine;

#endregion

namespace Microsoft.Ceres.Optimizer.Rules
{
    /// <summary>
    /// Abstract class for binary logical rules. Contains convenience properties
    /// and sharing update code.
    /// </summary>
    public abstract class BinaryLogicalRule : AbstractSearchRule
    {
        public BinaryLogicalRule(QueryOptimization queryOptimization)
            : base(queryOptimization)
        {
        }

        public BitSet RequiredLeft
        {
            get { return Required[0].BitSet; }
        }

        public BitSet RequiredRight
        {
            get { return Required[1].BitSet; }
        }

        public override void UpdatePlan(Plan plan)
        {
            UpdateSharing(plan);
        }

        public virtual void UpdateSharing(Plan plan)
        {
            plan.Sharing = (BitSet) plan.LeftChild.Sharing.Clone();
            plan.Sharing.Or(plan.RightChild.Sharing);

            if (queryOptimization.ShareEquivalenceClasses.ContainsKey(this)
                && plan.LeftChild.Shared && plan.RightChild.Shared)
            {
                plan.Sharing.Or(
                    queryOptimization.BitSetManager.GetWithValues(new NodeAttribute(Nodes.Single(), null).PropertyName));
                plan.Shared = true;
            }
        }
    }
}