﻿namespace Microsoft.Ceres.Optimizer.Rules.Infrastructure
{
    /// <summary>
    /// Sets the operator's ``behaviour''  ---
    /// that is whether the operator changes the record structure, the amount
    /// of tuples, its ordering/grouping, etc. For example, a selection
    /// operator changes the tuple amount, but not the ordering. A map
    /// operator can change the record structure, but not the amount or ordering.
    /// A group operator potentially changes both tuple amount, ordering,
    /// grouping and record structure.
    /// </summary>
    public interface IBehaviorMapper : IMapper
    {
        /// <summary>
        /// This method is called on the corresponding mapper for all nodes in the query in
        /// topological order and should set the behavior for the node.
        /// </summary>
        /// <param name="node">The node to set field behavior for.</param>
        void SetBehaviour(OperatorNode node);
    }
}