﻿#region

using System.Collections.Generic;
using Microsoft.Ceres.Optimizer.Engine;

#endregion

namespace Microsoft.Ceres.Optimizer.Rules.Infrastructure
{
    /// <summary>
    /// Producer rule produce properties and may require input. IBaseRule and ISearchRule inherit this interface.
    /// </summary>
    public interface IProducerRule : IRule
    {
        /// <summary>
        /// Properties produced by this rule instance.
        /// </summary>
        BitSetProxy Produced { get; set; }
        /// <summary>
        /// Filter used by the optimizer to quickly check if this rule is applicable to the
        /// goal searched for. The filter must be a subset of the goal for the rule to be used.
        /// </summary>
        BitSet Filter { get; }
        /// <summary>
        /// Unique Id for this rule instance.
        /// </summary>
        int Id { get; set; }
        /// <summary>
        /// The nodes in the input query that this rule was instantiated from.
        /// </summary>
        IList<OperatorNode> Nodes { get; set; }
        /// <summary>
        /// Determines if this and the other rule is structurally identical, i.e. if they have
        /// the same configured properties, the same join keys etc.
        /// </summary>
        /// <param name="other">Rule to compare with.</param>
        /// <returns>If the rules are structurally identical.</returns>
        bool StructurallyIdentical(IProducerRule other);

        /// <summary>
        /// This method will be called if this rule is determined to be share equivalent with another rule.
        /// It should map any attributes the other rule produces to be share equivalent to the corresponding attributes
        /// produced by this rule.
        /// </summary>
        /// <param name="from">The rule to map attributes from.</param>
        /// <param name="equivalenceClassAttributeMappings">The dictionary the mappings should be added to.</param>
        void MapShareEquivalentAttributes(IProducerRule from,
                                          Dictionary<NodeAttribute, NodeAttribute> equivalenceClassAttributeMappings);
    }
}