﻿#region

using Microsoft.Ceres.Optimizer.Engine;

#endregion

namespace Microsoft.Ceres.Optimizer.Rules.Infrastructure
{
    /// <summary>
    /// Maps the origins of attributes used in
    /// operators. The purpose is to always know where an attribute
    /// originates, to be able to correctly map operator dependencies. Also,
    /// attributes in expressions (such as in Map and
    /// Select) are altered to unambiguously refer to the attribute
    /// of a certain node using NodeAttribute instances instead of names.
    /// </summary>
    public interface IProducesMapper : IMapper
    {
        /// <summary>
        /// This method is called on the corresponding mapper for all nodes in the query in
        /// topological order and should set the produced attributes
        /// using OperatorNode's RecordDescriptor property. It should also convert
        /// all field name references to NodeAttribute.
        /// </summary>
        /// <param name="queryOptimization">The current instance of QueryOptimization.</param>
        /// <param name="node">The node to set produced attributes for.</param>
        void SetProduces(QueryOptimization queryOptimization, OperatorNode node);
    }
}