﻿#region

using Microsoft.Ceres.Optimizer.Engine;

#endregion

namespace Microsoft.Ceres.Optimizer.Rules.Infrastructure
{
    /// <summary>
    /// Common interface for all types of rules; helper, search or base rules.
    /// </summary>
    public interface IRule
    {
        /// <summary>
        /// Updates the given plan with information from this rule, like
        /// costs, order and so on.
        /// </summary>
        /// <param name="plan">The plan (constructed by this rule).</param>
        void UpdatePlan(Plan plan);

        /// <summary>Builds the physical algebra node for the given plan recursively.</summary>
        /// <param name="plan">Plan to build node for</param>
        /// <returns>The physical algebra node</returns>
        OperatorNode BuildAlgebra(Plan plan);

        /// <summary>
        /// Return the cost for the subtree with this rule on the top,
        /// using the cost algorithms for DAGs.
        /// </summary>
        /// <param name="plan">The plan (constructed by this rule).</param>
        /// <param name="reads">How many reads of the plan.</param>
        /// <returns>The costs for the subtree.</returns>
        ICost DagCosts(Plan plan, int reads);
    }
}