﻿#region

using System.Collections.Generic;
using Microsoft.Ceres.Optimizer.Engine;
using Microsoft.Ceres.Optimizer.GraphMatching;

#endregion

namespace Microsoft.Ceres.Optimizer.Rules.Infrastructure
{
    /// <summary>
    /// A Rule binder provides rule instances for part of a query. All rule binders must be tagged with
    /// [RuleBinder] and implement this interface.
    /// </summary>
    public interface IRuleBinder
    {
        /// <summary>The pattern in the input query this rule binder will instantiate rules for.</summary>
        AbstractNodeMatcher Pattern { get; }

        /// <summary>
        /// This method is called by the optimizer to get the rules this rule binder produces.
        /// </summary>
        /// <param name="queryOptimization">The currrent QueryOptimization instance.</param>
        /// <param name="matches">IEnumerable of matches to the pattern declared by the rule binder.</param>
        /// <returns>IEnumerable of rules that should be added to the search by the optimizer.</returns>
        IEnumerable<IProducerRule> GetRules(QueryOptimization queryOptimization, IEnumerable<OperatorNode> matches);
    }
}