﻿#region

using System.Collections.Generic;
using Microsoft.Ceres.Optimizer.Engine;

#endregion

namespace Microsoft.Ceres.Optimizer.Rules.Infrastructure
{
    /// <summary>
    /// Search rules are internal nodes in the query graph and participate actively in searching for a goal.
    /// Hence, they have 
    /// </summary>
    public interface ISearchRule : IProducerRule
    {
        /// <summary>
        /// List of required property sets with one item for each input, left to right.
        /// </summary>
        IList<BitSetProxy> Required { get; }
        /// <summary>Determines if this search rule is relevant to produce the given goal.</summary>
        /// <param name="goal">The goal searched for.</param>
        /// <returns>True if relevant, false otherwise.</returns>
        bool IsRelevantTo(BitSet goal);

        /// <summary>
        /// This method is called by the plan generator to have this rule produce plans
        /// that satisfy the goal properties set on the given plan set.
        /// </summary>
        /// <param name="planSet">Plan set describing the goal searched for. Resulting
        /// plan should be added to this plan set.</param>
        /// <param name="limit">Cost limit for the search. If the search exceeds this cost
        /// in the search for plan, that search branch should be aborted.</param>
        void Search(PlanSet planSet, ICost limit);
    }
}