﻿#region

using System.Collections.Generic;
using System.Linq;
using Microsoft.Ceres.Evaluation.Operators.BuiltIn.Join;
using Microsoft.Ceres.Optimizer.Engine;
using Microsoft.Ceres.Optimizer.Engine.Orders;
using Microsoft.Ceres.Optimizer.Rules.Infrastructure;

#endregion

namespace Microsoft.Ceres.Optimizer.Rules
{
    /// <summary>
    /// Helper rule for JoinRule, offering cost calculation and
    /// physical algebra building.
    /// </summary>
    public class MergeJoinRule : AbstractRule, IHelperRule
    {
        private const double INSTRUCTION_COUNT = 20;
        private JoinRule parent;

        public MergeJoinRule(QueryOptimization queryOptimization, JoinRule parent)
            : base(queryOptimization)
        {
            this.parent = parent;
        }

        public HashSet<Dependency> InducedDependencies { get; set; }

        #region IHelperRule Members

        public override void UpdatePlan(Plan plan)
        {
            base.UpdatePlan(plan);
            plan.OrderingState =
                plan.LeftChild.OrderingState.Apply(InducedDependencies).Apply(plan.RightChild.OrderingState);
        }

        public override OperatorNode BuildAlgebra(Plan plan)
        {
            OperatorNode node;
            if (GetOrCreatePhysicalNode(plan, typeof (MergeJoinOperator), out node))
            {
                node.Properties = parent.Nodes.Single().Properties;
                node["JoinPrefix"] = 1;
                node["RequiredPrefix"] = 1;
                parent.SwapJoinKeyIfNecessary((IList<NodeAttribute>)node["JoinKey"], plan);
                node.Rules = parent.Nodes.Single().Rules;
            }
            return node;
        }

        #endregion

        public override string ToString()
        {
            return "MergeJoin";
        }

        protected override BasicCost Cost(Plan plan)
        {
            BasicPlanSetState leftState = (BasicPlanSetState) plan.LeftChild.PlanSet.State;
            BasicPlanSetState rightState = (BasicPlanSetState) plan.RightChild.PlanSet.State;
            double cost = BasicCost.CpuCosts(parent.GetCardinality(leftState, rightState), INSTRUCTION_COUNT);
            return new BasicCost(cost, cost);
        }
    }
}