﻿#region

using System.Collections.Generic;
using Microsoft.Ceres.Optimizer.Engine;
using Microsoft.Ceres.Optimizer.Engine.Orders;

#endregion

namespace Microsoft.Ceres.Optimizer
{
    /// <summary>
    /// Stub a very basic system catalogue to be able to do basic cardinality- and selectivity estimates.
    /// </summary>
    public class SystemMetadataMock : ISystemMetadata
    {
        private readonly Dictionary<string, OrderDescription> indexOrder = new Dictionary<string, OrderDescription>();

        private readonly Dictionary<Pair<string, string>, RecordDescriptor> indexSchema =
            new Dictionary<Pair<string, string>, RecordDescriptor>();

        private readonly Dictionary<Pair<string, string>, BasicPlanSetState> indexStats =
            new Dictionary<Pair<string, string>, BasicPlanSetState>();

        #region ISystemMetadata Members

        public BasicPlanSetState GetIndexStats(string indexName, string lookupWord)
        {
            BasicPlanSetState state;
            if (indexStats.TryGetValue(new Pair<string, string>(indexName, lookupWord), out state))
                return state;
            if (indexStats.TryGetValue(new Pair<string, string>(indexName, null), out state))
                return state;

            throw new OptimizerException("Index state not found: Index: {0}, Word: {1}", indexName, lookupWord);
        }

        public RecordDescriptor GetIndexSchema(string indexName, string lookupWord)
        {
            RecordDescriptor result;
            if (indexSchema.TryGetValue(new Pair<string, string>(indexName, lookupWord), out result))
                return result;
            if (indexSchema.TryGetValue(new Pair<string, string>(indexName, null), out result))
                return result;

            throw new OptimizerException("Index schema not found: Index: {0}, Word: {1}", indexName, lookupWord);
        }

        public OrderDescription GetIndexOrder(string indexName)
        {
            OrderDescription desc;
            if (indexOrder.TryGetValue(indexName, out desc))
                return desc;

            throw new OptimizerException("Index order not found: Index: {0}", indexName);
        }

        #endregion

        public void SetIndexState(string indexName, string lookupWord, BasicPlanSetState state)
        {
            indexStats[new Pair<string, string>(indexName, lookupWord)] = state;
        }

        public void SetIndexSchema(string indexName, string lookupWord, RecordDescriptor schema)
        {
            indexSchema[new Pair<string, string>(indexName, lookupWord)] = schema;
        }

        public void SetIndexOrder(string indexName, OrderDescription order)
        {
            indexOrder[indexName] = order;
        }
    }
}