﻿#region

using System;
using System.Collections.Generic;

#endregion

namespace Microsoft.Ceres.Optimizer
{
    /// <summary>
    /// Validation utility methods for method parameters.
    /// </summary>
    internal static class Validation
    {
        public static void NoNulls<T>(string name, IEnumerable<T> values)
        {
            if (values == null)
                throw new ArgumentNullException(name);
            foreach (T t in values)
                if (t == null)
                    throw new ArgumentNullException(name + "[i]");
        }

        public static void NoNullsOrEmpty(string name, IEnumerable<string> values)
        {
            if (values == null)
                throw new ArgumentNullException(name);
            foreach (string s in values)
                NotNullOrEmtpy(name + "[i]", s);
        }

        public static void NoNulls(params Object[] values)
        {
            foreach (Object o in values)
                if (o == null)
                    throw new ArgumentNullException("Expected all values to be non-null.");
        }

        public static void NotNull(string name, object value)
        {
            if (value == null)
                throw new ArgumentNullException(name);
        }

        public static void NotNullOrEmtpy(string name, string value)
        {
            if (value == null)
                throw new ArgumentNullException(name);
            if (value.Length == 0)
                throw new ArgumentException(string.Format("{0} cannot be a zero-length string.", name));
        }

        public static void NotEquals(string name, object value, object forbidden)
        {
            if (value == forbidden)
                throw new ArgumentException(string.Format("Cannot be {0}.", forbidden), name);
        }

        public static void Assert(string name, bool value, string message)
        {
            if (!value)
                throw new ArgumentException(message, name);
        }
    }
}