﻿#region

using System.Collections.Generic;
using Microsoft.Ceres.Optimizer.GraphMatching;
using NUnit.Framework;

#endregion

namespace Microsoft.Ceres.Optimizer.Tests.GraphMatching
{
    /// <summary>
    /// Tests for GraphSearcher.
    /// </summary>
    [TestFixture]
    public class GraphSearcherTests
    {
        #region Setup/Teardown

        [SetUp]
        public void SetUp()
        {
            /* A <- B <- C <- D
             * ^ -------------/
             */

            a = new Node("A");
            b = new Node("B");
            c = new Node("C");
            d = new Node("D");

            any = new Node();

            a.AddChild(b);
            b.AddChild(c);
            c.AddChild(d);
            a.AddChild(d);

            aMatcher = new NamedNodeMatcher("A");
            bMatcher = new NamedNodeMatcher("B");
            cMatcher = new NamedNodeMatcher("C");
            dMatcher = new NamedNodeMatcher("D");
        }

        #endregion

        private Node a, b, c, d, any;
        private NamedNodeMatcher aMatcher, bMatcher, cMatcher, dMatcher;

        [Test]
        public void TestGraphSearcherWithSimplePattern()
        {
            // Search the graph (i.e. root and children) for B nodes. 
            GraphSearcher pattern = Match.NodeWithName("B").GroupAs("B").GetSearcher();
            pattern.AssertMatches(a);
            Assert.AreEqual(new List<Node>() {b}, pattern.Groups["B"]);

            pattern.AssertMatches(b);
            Assert.AreEqual(new List<Node>() {b, b}, pattern.Groups["B"]);
            pattern.AssertNotMatches(c, d);
        }

        [Test]
        public void TestMergeTrimSortPattern()
        {
            Node query, trim, sort, map, select;
            query = new Node("query");
            trim = new Node("trim");
            sort = new Node("sort");
            map = new Node("map");
            select = new Node("select");

            query.AddChild(trim);
            trim.AddChild(map);
            map.AddChild(sort);

            var pattern = Match.NodeWithName("trim").GroupAs("trim").WithDescendant(
                new AllMatcher(new ExpressionMatcher(node => node.Name != "select").WithSingleParent()).GroupAs("middle"),
                Match.NodeWithName("sort").GroupAs("sort")
                ).GetSearcher();

            pattern.AssertMatches(query);
            Assert.AreEqual(new List<Node>() {trim}, pattern.Groups["trim"]);
            Assert.AreEqual(new List<Node>() {map}, pattern.Groups["middle"]);
            Assert.AreEqual(new List<Node>() {sort}, pattern.Groups["sort"]);

            map.ReplaceWith(select);
            pattern.AssertNotMatches(query);

            select.Extract();
            pattern.AssertMatches(query);
        }
    }
}