﻿#region

using System.Collections.Generic;
using Microsoft.Ceres.Optimizer.Engine;
using Microsoft.Ceres.Optimizer.Operators;
using Microsoft.Ceres.Optimizer.Preprocessors;
using NUnit.Framework;

#endregion

namespace Microsoft.Ceres.Optimizer.Tests.Preprocessors
{
    [TestFixture]
    public class LogicalJoinTransformerTests : OptimizationTestBase
    {
        #region Setup/Teardown

        [SetUp]
        public void SetUp()
        {
            base.SetUp();
            preprocessor = new TransformationProcessor<PreprocessorAttribute>(queryOptimization);
            preprocessor.TransformationChain = new List<ITransformationRule>() {new LogicalJoinTransformer()};
        }

        #endregion

        private TransformationProcessor<PreprocessorAttribute> preprocessor;

        private static void assertResult(TestNode query)
        {
            TestNode topMostJoin = (TestNode) query.FirstChild;
            Assert.AreEqual(typeof (JoinOperator), topMostJoin.OperatorType);
            Assert.AreEqual(new List<string>() {"DocumentId0", "DocumentId2"}, topMostJoin["JoinKey"]);

            TestNode bottomMostJoin = (TestNode) query.FirstChild.FirstChild.FirstChild;
            Assert.AreEqual(typeof (JoinOperator), bottomMostJoin.OperatorType);
            Assert.AreEqual(new List<string>() {"DocumentId0", "DocumentId1"}, bottomMostJoin["JoinKey"]);
        }

        [Test]
        public void TestHybridHashJoin()
        {
            TestNode query = GetOutput(
                GetJoin("DocumentId0", "DocumentId2", 0.001,
                        GetSelection("DocumentId0 = 50", 0.001, 10.00001,
                                     GetJoin("DocumentId0", "DocumentId1", 0.006,
                                             GetLookup("word", "Mock0"),
                                             GetLookup("word", "Mock1"))),
                        GetLookup("word", "Mock2")));
            preprocessor.Transform(query);
            assertResult(query);
        }

        [Test]
        public void TestMergeJoin()
        {
            InitializeIndexStats(new double[] {1000, 2000, 3000});
            TestNode query = GetOutput(
                GetMergeJoin(new List<string>() {"DocumentId0", "DocumentId2"}, 0.001,
                             GetSelection("DocumentId0 = 50", 0.001, 10.00001,
                                          GetMergeJoin(new List<string>() {"DocumentId0", "DocumentId1"}, 0.006,
                                                       GetLookup("word", "Mock0"),
                                                       GetLookup("word", "Mock1"))),
                             GetLookup("word", "Mock2")));
            preprocessor.Transform(query);
            assertResult(query);
        }
    }
}