﻿#region

using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Ceres.Evaluation.Operators;
using Microsoft.Ceres.Evaluation.Operators.BuiltIn.Aggregation;
using Microsoft.Ceres.Evaluation.Operators.BuiltIn.Aggregation.BasicAggregators;
using Microsoft.Ceres.Evaluation.Operators.BuiltIn.Basic;
using Microsoft.Ceres.Optimizer.Engine;
using Microsoft.Ceres.Optimizer.Engine.Orders;
using Microsoft.Ceres.Optimizer.Preprocessors;
using NUnit.Framework;

#endregion

namespace Microsoft.Ceres.Optimizer.Tests.Preprocessors
{
    [TestFixture]
    public class OrderingExtractorTests : OptimizationTestBase
    {
        #region Setup/Teardown

        [SetUp]
        public override void SetUp()
        {
            base.SetUp();
            preprocessor = new TransformationProcessor<PreprocessorAttribute>(queryOptimization);
            preprocessor.TransformationChain = new List<ITransformationRule>() {new OrderingExtracter()};
        }

        #endregion

        private TransformationProcessor<PreprocessorAttribute> preprocessor;

        [Test]
        public void TestNegativeMatch()
        {
            TestNode query = GetOutput(
                GetMap(true,
                       new Dictionary<IdentifierProperty, ExpressionProperty>()
                           {
                               {new IdentifierProperty("CoolStuff"), new ExpressionProperty("DocumentId")}
                           },
                       new Dictionary<string, Type>() {{"CoolStuff", typeof (long)}},
                       GetGroup(new List<string>() {"DocumentId"}, true,
                                new Dictionary<string, Aggregator>() {{"Position", new SumAggregator()}},
                                new Dictionary<string, Type>() {{"Position", typeof (long)}},
                                GetSort(0, 0, true, new List<SortDefinition>() {new SortDefinition("DocumentId", true)},
                                        GetLookup("the", "Occurrence1")))));

            preprocessor.Transform(query);

            Assert.AreEqual(new List<OrderDescription>() {null}, query.Properties["Orders"]);
        }

        [Test]
        public void TestPositiveMatch()
        {
            TestNode query = GetOutput(
                GetMap(true,
                       new Dictionary<IdentifierProperty, ExpressionProperty>()
                           {
                               {new IdentifierProperty("CoolStuff"), new ExpressionProperty("DocumentId")}
                           },
                       new Dictionary<string, Type>() {{"CoolStuff", typeof (long)}},
                       GetSort(0, 0, true, new List<SortDefinition>() {new SortDefinition("DocumentId", true)},
                               GetLookup("the", "Occurrence1"))));
            int lookupNodeId = query.OnlyInput.OnlyInput.OnlyInput.Id;

            queryOptimization.Preprocess(query);
            preprocessor.Transform(query);

            CollectionAssert.AreEqual(
                new OrderDescription(new OrderDescriptionItem("AT_" + lookupNodeId + "_DocumentId", true)),
                ((IList<OrderDescription>) query["Orders"]).Single());
        }
    }
}