﻿#region

using System.Collections.Generic;
using Microsoft.Ceres.Optimizer.GraphMatching;
using Microsoft.Ceres.Optimizer.Preprocessors;
using NUnit.Framework;

#endregion

namespace Microsoft.Ceres.Optimizer.Tests
{
    [TestFixture]
    public class TrimSortMergeTests : TestBase
    {
        [Test]
        public void TestFire()
        {
            Node root = new Node();

            Node trim = new TestTrimNode(15, 5);

            root.Children.Add(trim);
            trim.OnlyParent = root;

            Node sort = new TestSortNode(-1, 3);

            trim.Children.Add(sort);
            sort.OnlyParent = trim;

            // Stub a match.
            GraphSearcher graphSearcher = new GraphSearcher(new AnythingMatcher())
                                              {
                                                  Groups = new Dictionary<string, List<Node>>()
                                                               {
                                                                   {"trim", new List<Node>() {trim}},
                                                                   {"sort", new List<Node>() {sort}},
                                                               },
                                              };

            MergeTrimSort preprocessor = new MergeTrimSort();
            preprocessor.Fire(graphSearcher);

            // Verify that the trim operator was removed
            Assert.AreSame(root, sort.OnlyParent);
            Assert.AreSame(sort, root.Children[0]);

            // New offset is 5 + 3
            Assert.AreEqual(8, sort.Properties["offset"]);
            // New hitcount is taken from trim: 15
            Assert.AreEqual(15, sort.Properties["hitcount"]);
        }
    }

    public class TestSortNode : OperatorNode
    {
        public TestSortNode(int count, int offset)
        {
            OperatorType = typeof (TestSortNode);
            Properties["hitcount"] = count;
            Properties["offset"] = offset;
        }
    }

    public class TestTrimNode : OperatorNode
    {
        public TestTrimNode(int count, int offset)
        {
            OperatorType = typeof (TestTrimNode);
            Properties["hitcount"] = count;
            Properties["offset"] = offset;
        }
    }
}