﻿#region

using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using NUnit.Framework;

#endregion

namespace Microsoft.Ceres.Optimizer.Tests
{
    public class ParameterVisitor : ExpressionVisitor
    {
        public List<string> variables;

        public ParameterVisitor()
        {
            variables = new List<string>();
        }

        protected override Expression VisitParameter(ParameterExpression p)
        {
            variables.Add(p.ToString());
            return base.VisitParameter(p);
        }
    }

    [TestFixture]
    public class TestExpressionVisitors
    {
        #region Setup/Teardown

        [SetUp]
        public void SetUp()
        {
            isOdd = i => (i & 1) == 1;
        }

        #endregion

        private Expression<Func<int, bool>> isOdd;

        [Test]
        public void TestExpressionPostOrderIterator()
        {
            // Comparing to strings as that takes a lot less code.
            List<string> expected = new List<string>() {"i", "1", "(i & 1)", "1", "((i & 1) = 1)"};

            List<string> actual = new List<string>(from node in isOdd.IteratePostorder() select node.ToString());

            Assert.AreEqual(expected, actual);

            // Build the list with a simple visitor.

            actual = new List<string>();
            isOdd.TraversePostorder(e => { actual.Add(e.ToString()); });

            Assert.AreEqual(actual, expected);
        }

        [Test]
        public void TestExpressionVisitor()
        {
            Expression<Func<int, int, int, bool>> e = (a, b, c) => a == b || b == c;

            var visitor = new ParameterVisitor();
            visitor.Visit(e);

            var expected = new List<string>() {"a", "b", "b", "c"};
            Assert.AreEqual(expected, visitor.variables);

            Expression<Func<int, int, int>> max = (a, b) => a > b ? a : b;
            visitor = new ParameterVisitor();
            visitor.Visit(max);
            expected = new List<string>() {"a", "b", "a", "b"};
            Assert.AreEqual(expected, visitor.variables);
        }
    }
}