﻿#region

using System;
using System.Collections.Generic;
using NUnit.Framework;

#endregion

namespace Microsoft.Ceres.Optimizer.Tests
{
    public delegate void NodeValidatorDelegate(OperatorNode node);

    /// <summary>
    /// TestNode inherits from OperatorNode and adds the possibility
    /// to recursively assert equality between node graphs.
    /// </summary>
    public class TestNode : OperatorNode
    {
        public TestNode(Type operatorType) : base(operatorType)
        {
        }

        /// <summary>Custom validator for this exact node.</summary>
        public NodeValidatorDelegate CustomValidator { get; set; }

        /// <summary>
        /// Asserts that this and the specified node graph are equal recursively
        /// using NUnit asserts.
        /// </summary>
        /// <param name="other">The node grap to assert.</param>
        public void AreEqualRecursively(OperatorNode other)
        {
            if (OperatorType != null)
                Assert.AreEqual(OperatorType, other.OperatorType);

            if (Rules != null)
                CollectionAssert.AreEquivalent(Rules, other.Rules);

            if (Children != null)
                Assert.AreEqual(Children.Count, other.Children.Count);

            if (Properties != null)
                foreach (KeyValuePair<string, object> pair in Properties)
                    Assert.AreEqual(pair.Value, other[pair.Key], pair.Key);

            if (CustomValidator != null)
                CustomValidator(other);

            if (Children != null)
                for (int i = 0; i < Children.Count; i++)
                    ((TestNode) Children[i]).AreEqualRecursively((OperatorNode) other.Children[i]);
        }
    }
}