﻿#region

using System;
using NUnit.Framework;

#endregion

namespace Microsoft.Ceres.Optimizer.Tests
{
    /// <summary>
    /// Extensions to NUnit's Assert class.
    /// </summary>
    internal static class XAssert
    {
        /// <summary>
        /// Asserts that the given delegate throws and exception of the specified type.
        /// </summary>
        /// <typeparam name="T">The exact type of the exptected exceptions. Subclassed exceptions will not match.</typeparam>
        /// <param name="exceptionThrower">Delegate that is expected to throw exception.</param>
        /// <returns>The thrown exception, useful for further asserts.</returns>
        public static T Throws<T>(Action exceptionThrower) where T : Exception
        {
            try
            {
                exceptionThrower();
            }
            catch (Exception ex)
            {
                if (ex.GetType().Equals(typeof (T)))
                {
                    return (T) ex;
                }
                else
                {
                    throw new AssertionException(string.Format("Expected {0}, but got {1}.", typeof (T).Name,
                                                               ex.GetType().Name));
                }
            }
            throw new AssertionException(string.Format("Expected {0}, but none was thrown.", typeof (T).Name));
        }
    }
}