﻿#region

using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Ceres.Evaluation.Operators;

#endregion

namespace Microsoft.Ceres.Optimizer.Transformer
{
    /// <summary>
    /// Utility methods for working with MARS' data structures.
    /// </summary>
    public static class MarsExtensions
    {
        /// <summary>Get all the children of an operator.</summary>
        public static IEnumerable<OperatorBase> GetChildren(this OperatorBase op)
        {
            return from edge in op.Inputs select edge.Source;
        }

        /// <summary>Iterates through the operator graph in topological ordering.</summary>
        public static IEnumerable<OperatorBase> IterateTopologicalOrdering(this OperatorBase rootOperator)
        {
            HashSet<OperatorBase> discovered = new HashSet<OperatorBase>();
            HashSet<OperatorBase> explored = new HashSet<OperatorBase>();

            Stack<OperatorBase> stack = new Stack<OperatorBase>();
            stack.Push(rootOperator);
            // Use a stack and the standard DFS way of producing a topological order.
            while (stack.Count > 0)
            {
                OperatorBase node = stack.Peek();
                if (explored.Contains(node))
                {
                    stack.Pop();
                    continue;
                }
                discovered.Add(node);

                List<OperatorBase> newNodes = new List<OperatorBase>();
                foreach (var child in node.GetChildren())
                {
                    if (!explored.Contains(child))
                    {
                        if (discovered.Contains(child))
                        {
                            throw new InvalidOperationException("Graph contains cycle.");
                        }
                        newNodes.Add(child);
                    }
                }

                if (newNodes.Count > 0)
                {
                    foreach (var newNode in newNodes)
                    {
                        stack.Push(newNode);
                    }
                }
                else
                {
                    explored.Add(node);
                    yield return node;
                }
            }
        }
    }
}