﻿#region

using Microsoft.Ceres.Evaluation.DataModel;
using Microsoft.Ceres.Evaluation.DataModel.Types;
using Microsoft.Ceres.Evaluation.Operators.Parsing;
using Microsoft.Ceres.Evaluation.Processing;
using Microsoft.Ceres.Evaluation.Processing.Producers;
using Microsoft.Ceres.Optimizer.Engine;
using Microsoft.Ceres.Optimizer.Engine.Orders;
using Microsoft.Ceres.SearchCore.Services.SystemCatalog;

#endregion

namespace Microsoft.Ceres.Optimizer.Transformer
{
    /// <summary>
    /// Evaulator for the optimizer query chain operator,
    /// which performs the optimizer invocation.
    /// </summary>
    public class OptimizerTransformerEvaluator : ProducerEvaluator<OptimizerTransformerOperator>
    {
        public IOperatorDecorator OperatorDecorator { get; set; }
        public ISystemCatalog Catalog { get; set; }

        protected override IRecordProducer GetProducer(OptimizerTransformerOperator op, IRecordSetTypeDescriptor type,
                                                       IEvaluationContext context)
        {
            // TODO: Do this properly with injection, interface, etc.
            OptimizerTransformerProducer producer = new OptimizerTransformerProducer();

            QueryOptimization optimizer = new QueryOptimization {PrintStats = true, SystemMetadata = GetIndexStats()};

            producer.OptimizerFacade = new OptimizerFacade()
                                           {
                                               OperatorDecorator = OperatorDecorator,
                                               Catalog = Catalog,
                                               Optimizer = optimizer
                                           };
            return producer;
        }

        // We mock the system catalog for now.
        private SystemMetadataMock GetIndexStats()
        {
            SystemMetadataMock metadata = new SystemMetadataMock();
            metadata.SetIndexState("Occurrence1", "search", new BasicPlanSetState(10000, 100));
            metadata.SetIndexState("Occurrence1", "engine", new BasicPlanSetState(5000, 100));
            metadata.SetIndexState("Occurrence1", "olstad", new BasicPlanSetState(500, 100));
            metadata.SetIndexState("Occurrence1", "information", new BasicPlanSetState(2000, 100));
            metadata.SetIndexState("Occurrence1", null, new BasicPlanSetState(500, 100));
            metadata.SetIndexSchema("Occurrence1", null,
                                    new RecordDescriptor(
                                        new Field(new NodeAttribute(null, "DocumentId"), typeof (long)),
                                        new Field(new NodeAttribute(null, "Position"), typeof (long)),
                                        new Field(new NodeAttribute(null, "Scope"), typeof (string))));
            metadata.SetIndexOrder("Occurrence1", new OrderDescription(
                                                      new OrderDescriptionItem("DocumentId", true),
                                                      new OrderDescriptionItem("Position", true)));

            metadata.SetIndexState("Occurrence2", "DocumentName", new BasicPlanSetState(8000, 35));
            metadata.SetIndexSchema("Occurrence2", "DocumentName",
                                    new RecordDescriptor(
                                        new Field(new NodeAttribute(null, "DocumentId"), typeof (long)),
                                        new Field(new NodeAttribute(null, "DocumentName"), typeof (string))));
            metadata.SetIndexState("Occurrence2", "DocumentType", new BasicPlanSetState(8000, 10));
            metadata.SetIndexSchema("Occurrence2", "DocumentType",
                                    new RecordDescriptor(
                                        new Field(new NodeAttribute(null, "DocumentId"), typeof (long)),
                                        new Field(new NodeAttribute(null, "DocumentType"), typeof (string))));
            metadata.SetIndexOrder("Occurrence2", new OrderDescription(
                                                      new OrderDescriptionItem("DocumentId", true)));

            return metadata;
        }
    }

    /// <summary>
    /// Producer which invokes the optimizer and sets the optimized query back into MARS.
    /// </summary>
    public class OptimizerTransformerProducer : SingleProducerBase
    {
        // TODO: Do this properly with injection, interface, etc.
        public OptimizerFacade OptimizerFacade { get; set; }

        public override void ProcessRecord(IRecord record)
        {
            if (OptimizerFacade == null)
            {
                SetNextRecord(record);
                return;
            }

            SetNextRecord(OptimizerFacade.Optimize(record));
        }
    }
}